/*
 * gnome-open
 *
 * Copyright (C) 2007 Ryan Hughes
 *
 * Gnome-open is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * Gnome-open is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Authors: Ryan Hughes <ryan@iheartryan.com>
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libgnomevfs/gnome-vfs-mime.h>
#include <libgnomevfs/gnome-vfs-mime-handlers.h>

/**
 * Get the default application, given a uri.
 */
GnomeVFSMimeApplication *
mime_get_default_application_for_uri (char *uri)
{
  GnomeVFSMimeApplication *app;
  char *mime_type;

	mime_type = gnome_vfs_get_mime_type(uri);

  app = gnome_vfs_mime_get_default_application_for_uri (uri, mime_type);

  g_free (mime_type);

  return app;
}


/**
 * Given a path (relative or full), return a newly-allocated text uri.
 */
char *
uri_from_path(char *path)
{
  char *filepath;
  char *uri;
  char *pwd;
  int len;

  if (path[0] != '/') {
    pwd = getenv("PWD");
    len = strlen(pwd)+strlen(path)+1;
    filepath = (char *)malloc(sizeof(char) * (len+1));
    filepath[0] = 0;
    strcat(filepath, pwd);
    strcat(filepath, "/");
    strcat(filepath, path);
  } // if it's a relative path
  else {
    filepath = strdup(path);
  } // else it's an absolute path

  uri = gnome_vfs_get_uri_from_local_path(filepath);

  g_free(filepath);

  return uri;
}


int 
main(int argc, 
     char **argv)
{
  char *fileuri;
  GnomeVFSMimeApplication *app;
  GList *openthis;
  GnomeVFSResult result;

  // One day, I'm going to use these to open multiple files per app in a single
  // call.
  int a;
  // GData *appz = NULL;
  // GData *filez_by_app = NULL;

  if (argc < 2) {
    fprintf(stderr, 
        "Usage: %s <file> [<file> ...].\n<file>s will open, as if clicked in gnome.\n",
        argv[0]);
    exit(1);
  } // if args wrong

  gnome_vfs_init();

  for (a=1; a<argc; a++) {
    fileuri = uri_from_path(argv[a]);
    app = mime_get_default_application_for_uri (fileuri);

    // Check if it can open multiple.  Remember!
    // Whoops.  It'll take a bit of doing to get it to do multiple, because you
    // could pass in files of different types.  It has to group them, so that
    // it's got all the ones that are opened by apps that can open multiple, so
    // that they can open multiple, and then all the other ones, so they can
    // open singly.
    // This requires some sort of data structure, like a hash table storing
    // lists or something.
    // For now, we'll only support opening one file.
   
    openthis = g_list_append(NULL, fileuri);
    result = gnome_vfs_mime_application_launch(app, openthis);

    switch (result) {
    case GNOME_VFS_ERROR_NOT_SUPPORTED:
      fprintf(stderr,
          "%s: The application %s does not know how to open the file %s.\n",
          argv[0],
          app->name,
          fileuri);
      break;
    case GNOME_VFS_ERROR_PARSE:
      fprintf(stderr,
          "%s: Error parsing command for %s.  File %s not opened.\n",
          argv[0],
          app->name,
          fileuri);
      break;
    case GNOME_VFS_ERROR_LAUNCH:
      fprintf(stderr,
          "%s: %s could not be launched.  File %s not opened.\n",
          argv[0],
          app->name,
          fileuri);
      break;
    case GNOME_VFS_ERROR_INTERNAL:
      fprintf(stderr,
          "%s: Unknown error.  %s could not be launched.  File %s not opened.\n",
          argv[0],
          app->name,
          fileuri);
      break;
    }

    g_free(fileuri);
    g_free(app);
  } // for all the args

  gnome_vfs_shutdown();
} // int main
